@@ -1,7 +1,7 @@
 /*
 	dom.c[pp]
 
-	gSOAP DOM implementation v3
+	gSOAP DOM implementation v4
 
 gSOAP XML Web services tools
 Copyright (C) 2000-2012, Robert van Engelen, Genivia, Inc. All Rights Reserved.
@@ -114,7 +114,7 @@ SOAP_FMAC3
 void
 SOAP_FMAC4
 soap_traverse_xsd__anyType(struct soap *soap, struct soap_dom_element *node, const char *s, soap_walker p, soap_walker q)
-{ 
+{ (void)soap; (void)node; (void)s; (void)p; (void)q; 
 }
 
 /******************************************************************************/
@@ -123,14 +123,14 @@ SOAP_FMAC1
 void
 SOAP_FMAC2
 soap_serialize_xsd__anyAttribute(struct soap *soap, const struct soap_dom_attribute *node)
-{
+{ (void)soap; (void)node;
 }
 
 SOAP_FMAC1
 void
 SOAP_FMAC2
 soap_traverse_xsd__anyAttribute(struct soap *soap, struct soap_dom_attribute *node, const char *s, soap_walker p, soap_walker q)
-{
+{ (void)soap; (void)node; (void)s; (void)p; (void)q; 
 }
 
 /******************************************************************************/
@@ -256,7 +256,8 @@ SOAP_FMAC1
 int
 SOAP_FMAC2
 soap_out_xsd__anyType(struct soap *soap, const char *tag, int id, const struct soap_dom_element *node, const char *type)
-{ if (node)
+{ (void)id; (void)type;
+  if (node)
   { const char *prefix; /* namespace prefix, if namespace is present */
     size_t colon;
     if (!(soap->mode & SOAP_DOM_ASIS))
@@ -403,7 +404,8 @@ SOAP_FMAC1
 int
 SOAP_FMAC2
 soap_out_xsd__anyAttribute(struct soap *soap, const char *tag, int id, const struct soap_dom_attribute *node, const char *type)
-{ if (!(soap->mode & SOAP_DOM_ASIS))
+{ (void)tag; (void)id; (void)type;
+  if (!(soap->mode & SOAP_DOM_ASIS))
   { const struct soap_dom_attribute *att;
     for (att = node; att; att = att->next)
     { if (att->name && att->data && !strncmp(att->name, "xmlns:", 6))
@@ -440,8 +442,9 @@ SOAP_FMAC1
 struct soap_dom_element *
 SOAP_FMAC2
 soap_in_xsd__anyType(struct soap *soap, const char *tag, struct soap_dom_element *node, const char *type)
-{ register struct soap_attribute *tp;
-  register struct soap_dom_attribute **att;
+{ struct soap_attribute *tp;
+  struct soap_dom_attribute **att;
+  (void)tag; (void)type;
   if (soap_peek_element(soap))
   { if (soap->error != SOAP_NO_TAG)
       return NULL;
@@ -452,7 +455,7 @@ soap_in_xsd__anyType(struct soap *soap, const char *tag, struct soap_dom_element
       }
     }
     soap_default_xsd__anyType(soap, node);
-    if (!(node->data = soap_string_in(soap, 1, -1, -1)) || !*node->data)
+    if (!(node->data = soap_string_in(soap, 1, -1, -1, NULL)) || !*node->data)
       return NULL;
     DBGLOG(TEST, SOAP_MESSAGE(fdebug, "DOM node with cdata\n"));
     soap->error = SOAP_OK;
@@ -537,10 +540,10 @@ soap_in_xsd__anyType(struct soap *soap, const char *tag, struct soap_dom_element
     else if (soap->error == SOAP_NO_TAG)
     { DBGLOG(TEST, SOAP_MESSAGE(fdebug, "DOM node '%s' has cdata\n", node->name));
       if ((soap->mode & SOAP_C_UTFSTRING) || (soap->mode & SOAP_C_MBSTRING))
-      { if (!(node->data = soap_string_in(soap, 1, -1, -1)))
+      { if (!(node->data = soap_string_in(soap, 1, -1, -1, NULL)))
           return NULL;
       }
-      else if (!(node->wide = soap_wstring_in(soap, 1, -1, -1)))
+      else if (!(node->wide = soap_wstring_in(soap, 1, -1, -1, NULL)))
         return NULL;
     }
     else
@@ -558,9 +561,10 @@ SOAP_FMAC1
 struct soap_dom_attribute *
 SOAP_FMAC2
 soap_in_xsd__anyAttribute(struct soap *soap, const char *tag, struct soap_dom_attribute *node, const char *type)
-{ register struct soap_attribute *tp;
+{ struct soap_attribute *tp;
   struct soap_dom_attribute *tmp = node;
   struct soap_dom_attribute *att = node;
+  (void)tag; (void)type;
   for (tp = soap->attributes; tp; tp = tp->next)
   { if (tp->visible)
     { if (!att)
@@ -635,7 +639,7 @@ soap_dom_next_attribute(struct soap_dom_attribute *att)
 
 static const char *
 soap_lookup_ns_prefix(struct soap *soap, const char *ns)
-{ register struct soap_nlist *np;
+{ struct soap_nlist *np;
   for (np = soap->nlist; np; np = np->next)
   { if (np->ns && !strcmp(np->ns, ns)) 
       return np->id;
@@ -647,7 +651,7 @@ soap_lookup_ns_prefix(struct soap *soap, const char *ns)
 
 static const char *
 soap_push_ns_prefix(struct soap *soap, const char *id, const char *ns, int flag)
-{ register struct soap_nlist *np;
+{ struct soap_nlist *np;
   if (!id)
   { struct Namespace *n;
     for (n = soap->local_namespaces; n && n->id; n++)
@@ -1092,6 +1096,7 @@ soap_dom_attribute_iterator &soap_dom_attribute_iterator::operator++()
 \******************************************************************************/
 
 #ifndef UNDER_CE
+#ifndef WITH_COMPAT
 
 std::ostream &operator<<(std::ostream &o, const struct soap_dom_element &e)
 { if (!e.soap)
@@ -1137,6 +1142,7 @@ std::istream &operator>>(std::istream &i, struct soap_dom_element &e)
   return i;
 }
 
+#endif
 #endif
 
 #endif
